package pomocov.utils;

import beast.util.Randomizer;
import pomocov.PoMoCoVTreeLikelihood;
import pomocov.PoMoGeneral;

public class ACGTCountsPartial {

    // For calculating sample partials
    private static class BiggestTwoCounts {
        final int count1;
        final int count2;
        final int nuc1;
        final int nuc2;
        final boolean debugTie; // set if randomisation used

        public BiggestTwoCounts(int count1, int count2, int nuc1, int nuc2, boolean debugTie) {
            this.count1 = count1;
            this.count2 = count2;
            this.nuc1 = nuc1;
            this.nuc2 = nuc2;
            this.debugTie = debugTie;
        }
    }


    public static void calcSamplePartialsForOnePattern(int [] acgtCounts, int vPop, double sequencingError, double[] partials, int patt) {
        int nStates = IndicesEtc.nPoMoStates(vPop);
        int k = patt * nStates;

        BiggestTwoCounts btc = findBiggestTwoCounts(acgtCounts);

        if (btc.nuc1 == -1) {
            for (int iState = 0; iState < nStates; iState++) {
                partials[k++] = 1.0;
            }
        } else {
            //fixed states
            for (int iState = 0; iState < 4; iState++) {
                if (iState == btc.nuc1) {
                    partials[k++] = sampProb(1.0 - sequencingError, btc.count1, sequencingError / 3.0, btc.count2);
                } else if (iState == btc.nuc2) {
                    partials[k++] = sampProb(sequencingError / 3.0, btc.count1, 1.0 - sequencingError, btc.count2);
                } else {
                    partials[k++] = sampProb(sequencingError / 3.0, btc.count1 + btc.count2, 1.0 - sequencingError, 0);
                }
            }
            //groups of polymorphisms
            for (int iGroup = 0; iGroup < 6; iGroup++) {
                int[] ind = PoMoGeneral.indeces(iGroup);
                for (int iState = 0; iState < vPop - 1; iState++) {
                    double[] probs = new double[2];
                    double freq;
                    if (btc.nuc1 == ind[0]) {
                        freq = (iState + 1.0) / ((double) (vPop));
                    } else if (btc.nuc1 == ind[1]) {
                        freq = (vPop - (iState + 1.0)) / ((double) (vPop));
                    } else {
                        freq = 0.0;
                    }
                    probs[0] = freq * (1.0 - sequencingError) + (1.0 - freq) * sequencingError / 3.0;
                    if (btc.nuc2 == ind[0]) {
                        freq = (iState + 1.0) / ((double) (vPop));
                    } else if (btc.nuc2 == ind[1]) {
                        freq = (vPop - (iState + 1.0)) / ((double) (vPop));
                    } else {
                        freq = 0.0;
                    }
                    probs[1] = freq * (1.0 - sequencingError) + (1.0 - freq) * sequencingError / 3.0;
                    partials[k++] = sampProb(probs[0], btc.count1, probs[1], btc.count2);
                }
            }
        }
    }


    public static boolean debugRandomisationRequired(int[] acgtCounts) {
        BiggestTwoCounts btc = findBiggestTwoCounts(acgtCounts);
        return btc.debugTie;
    }


    private static double sampProb(double p1, int v1, double p2, int v2) {
        double res = 1.0, bigP, smallP;
        int big, small;
        if (v1 > v2) {
            big = v1;
            bigP = p1;
            small = v2;
            smallP = p2;
        } else {
            big = v2;
            bigP = p2;
            small = v1;
            smallP = p1;
        }
        for (int i = 1; i <= small; i++) {
            res = res * (i + big) * smallP * bigP / i;
        }
        for (int i = 1; i <= big - small; i++) {
            res = res * bigP;
        }
        return res;
    }


    private static BiggestTwoCounts findBiggestTwoCounts(int[] acgtCounts) {
        assert acgtCounts.length == 4;
        boolean debugTie = false;

        int count1 = 0;
        int count2 = 0;
        int nuc1 = -1;
        int nuc2 = -1;
        for (int iNuc = 0; iNuc < 4; iNuc++) {
            if (acgtCounts[iNuc] > count1) {
                nuc2 = nuc1;
                count2 = count1;
                nuc1 = iNuc;
                count1 = acgtCounts[iNuc];
            } else if (acgtCounts[iNuc] > count2) {
                nuc2 = iNuc;
                count2 = acgtCounts[iNuc];
            }
        }
        if (count2 > 0) {
            int numSeconds = 0;
            boolean equalFirst = false;
            if (count2 == count1) equalFirst = true;
            for (int iNuc = 0; iNuc < 4; iNuc++) {
                if (acgtCounts[iNuc] == count2) numSeconds += 1;
            }
            if (numSeconds > 1) {
                // if there are multiple nucleotides sharing the second place of most abundant, then
                // pick one at random as second most abundant.
                int newSecond = Randomizer.nextInt(numSeconds);
                debugTie = true;
                int secondCount = 0;
                for (int iNuc = 0; iNuc < 4; iNuc++) {
                    if (acgtCounts[iNuc] == count2 && secondCount == newSecond) nuc2 = iNuc;
                    else secondCount += 1;
                }
                if (equalFirst) {
                    int newFirst = -1;
                    if (numSeconds == 2) {
                        newFirst = 1 - newSecond;
                    } else {
                        newFirst = Randomizer.nextInt(numSeconds - 1);
                        debugTie = true;
                        if (newFirst >= newSecond) newFirst += 1;

                    }
                    int firstCount = 0;
                    for (int iNuc = 0; iNuc < 4; iNuc++) {
                        if (acgtCounts[iNuc] == count2 && firstCount == newFirst) nuc1 = iNuc;
                        else firstCount += 1;
                    }
                }
            }
        }
        return new BiggestTwoCounts(count1, count2, nuc1, nuc2, debugTie);
    }




}
