package pomocov.utils;

public class IndicesEtc {


    // Indices for rates and mutation probabilities used in calcPartialsOneBranch_Linear(),
    public static final int rAC = 0;
    public static final int rAG = 1;
    public static final int rAT = 2;
    public static final int rCA = 3;
    public static final int rCG = 4;
    public static final int rCT = 5;
    public static final int rGA = 6;
    public static final int rGC = 7;
    public static final int rGT = 8;
    public static final int rTA = 9;
    public static final int rTC = 10;
    public static final int rTG = 11;

    // Indices for states. 4 pure states, used for any vPop.
    public static final int sAAA = 0;
    public static final int sCCC = 1;
    public static final int sGGG = 2;
    public static final int sTTT = 3;

    // When vPop = 3, there are also 6*2 mixed states which are named.
    public static final int sACC = 4; public static final int sAAC = 5;
    public static final int sAGG = 6; public static final int sAAG = 7;
    public static final int sATT = 8; public static final int sAAT = 9;
    public static final int sCGG = 10; public static final int sCCG = 11;
    public static final int sCTT = 12; public static final int sCCT = 13;
    public static final int sGTT = 14; public static final int sGGT = 15;

    // Indices for nucleotides
    public static final int nucleotideA = 0;
    public static final int nucleotideC = 1;
    public static final int nucleotideG = 2;
    public static final int nucleotideT = 3;

    public static int nPoMoStates(int vPop) {
        return 4 + 6 * (vPop - 1);
    }



    // These provide the first and last indices of the 6 sets of polymorphic states
    public static int stateA1Cn(int vPop) {
        return 4;
    }
    public static int stateAnC1(int vPop) { return 4 + 1 * (vPop-1) - 1; }

    public static int stateA1Gn(int vPop) { return 4 + 1 * (vPop-1); }
    public static int stateAnG1(int vPop) {
        return 4 + 2 * (vPop-1) - 1;
    }

    public static int stateA1Tn(int vPop) {
        return 4 + 2 * (vPop-1);
    }
    public static int stateAnT1(int vPop) {
        return 4 + 3 * (vPop-1) - 1;
    }

    public static int stateC1Gn(int vPop) {
        return 4 + 3 * (vPop-1);
    }
    public static int stateCnG1(int vPop) {
        return 4 + 4 * (vPop-1) - 1;
    }

    public static int stateC1Tn(int vPop) {
        return 4 + 4 * (vPop-1);
    }
    public static int stateCnT1(int vPop) {
        return 4 + 5 * (vPop-1) - 1;
    }

    public static int stateG1Tn(int vPop) {
        return 4 + 5 * (vPop-1);
    }
    public static int stateGnT1(int vPop) {
        return 4 + 6 * (vPop-1) - 1;
    }

    public static class EdgeIndices {
        public int nucX;
        public int nucY;
        public int sX1Yn;
        public int sXnY1;
    }

    public static EdgeIndices EdgeIndicesForAC(int vPop) {
        EdgeIndices eis = new EdgeIndices();
        eis.nucX = nucleotideA;
        eis.nucY = nucleotideC;
        eis.sX1Yn = stateA1Cn(vPop);
        eis.sXnY1 = stateAnC1(vPop);
        return eis;
    }

    public static EdgeIndices EdgeIndicesForAG(int vPop) {
        EdgeIndices eis = new EdgeIndices();
        eis.nucX = nucleotideA;
        eis.nucY = nucleotideG;
        eis.sX1Yn = stateA1Gn(vPop);
        eis.sXnY1 = stateAnG1(vPop);
        return eis;
    }

    public static EdgeIndices EdgeIndicesForAT(int vPop) {
        EdgeIndices eis = new EdgeIndices();
        eis.nucX = nucleotideA;
        eis.nucY = nucleotideT;
        eis.sX1Yn = stateA1Tn(vPop);
        eis.sXnY1 = stateAnT1(vPop);
        return eis;
    }


    public static EdgeIndices EdgeIndicesForCG(int vPop) {
        EdgeIndices eis = new EdgeIndices();
        eis.nucX = nucleotideC;
        eis.nucY = nucleotideG;
        eis.sX1Yn = stateC1Gn(vPop);
        eis.sXnY1 = stateCnG1(vPop);
        return eis;
    }

    public static EdgeIndices EdgeIndicesForCT(int vPop) {
        EdgeIndices eis = new EdgeIndices();
        eis.nucX = nucleotideC;
        eis.nucY = nucleotideT;
        eis.sX1Yn = stateC1Tn(vPop);
        eis.sXnY1 = stateCnT1(vPop);
        return eis;
    }


    public static EdgeIndices EdgeIndicesForGT(int vPop) {
        EdgeIndices eis = new EdgeIndices();
        eis.nucX = nucleotideG;
        eis.nucY = nucleotideT;
        eis.sX1Yn = stateG1Tn(vPop);
        eis.sXnY1 = stateGnT1(vPop);
        return eis;
    }





    // First 4 rows are fixed states, ordered A,C,G,T. Then are polymorhic state group, ordered A->C, A->G, A->T, C->G, C->T, G->T.
    // Within each group states are ordered 1A-9C, 2A-8C, etc...



}
